@@ -37,6 +37,7 @@ G_BEGIN_DECLS
 #define RB_STOCK_LIBRARY       "rhythmbox-library"
 #define RB_STOCK_IRADIO        "rhythmbox-iradio"
 #define RB_STOCK_PLAYLIST      "rhythmbox-playlist"
+#define RB_STOCK_PLAYLIST_MENU "rhythmbox-playlist-menu"
 #define RB_STOCK_PLAYING       "rhythmbox-playing"
 #define RB_STOCK_PAUSED        "rhythmbox-paused"
 #define RB_STOCK_ALBUM         "rhythmbox-album"
@@ -47,6 +48,7 @@ G_BEGIN_DECLS
 #define RB_STOCK_VOLUME_MUTE   "rhythmbox-volume-mute"
 #define RB_STOCK_TRAY_ICON     "rhythmbox-tray-icon"
 #define RB_STOCK_SET_STAR      "rhythmbox-set-star"
+#define RB_STOCK_AUTOMATIC_PLAYLIST "rhythmbox-automatic-playlist"
 #define RB_STOCK_UNSET_STAR    "rhythmbox-unset-star"
 #define RB_STOCK_NO_STAR       "rhythmbox-no-star"
 #define RB_STOCK_AUDIOCD       "rhythmbox-audiocd"
