@@ -159,7 +159,8 @@ rb_gvalue_compare (GValue *a, GValue *b)
 	case G_TYPE_OBJECT:
 	default:
 		g_assert_not_reached ();
+		retval = 0;
 		break;
 	}
-	return FALSE;
+	return retval;
 }
