@@ -38,6 +38,7 @@
 #include <libgnome/gnome-program.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include <libgnomeui/gnome-about.h>
+#include <libgnomeui/gnome-client.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 #include <string.h>
@@ -79,6 +80,12 @@
 #include "rb-thread-helpers.h"
 #include "eel-gconf-extensions.h"
 
+#ifdef WITH_DASHBOARD
+#include <glib.h>
+#include <sys/time.h>
+#include "dashboard.c"
+#endif /* WITH_DASHBOARD */
+
 static void rb_shell_class_init (RBShellClass *klass);
 static void rb_shell_init (RBShell *shell);
 static void rb_shell_finalize (GObject *object);
@@ -224,6 +231,18 @@ static gboolean rb_shell_show_popup_cb (RBSourceList *sourcelist,
 					RBShell *shell);
 static void tray_deleted_cb (GtkWidget *win, GdkEventAny *event, RBShell *shell);
 
+static gboolean save_yourself_cb (GnomeClient *client, 
+                                  gint phase,
+                                  GnomeSaveStyle save_style,
+                                  gboolean shutdown,
+                                  GnomeInteractStyle interact_style,
+                                  gboolean fast,
+                                  RBShell *shell);
+
+static void session_die_cb (GnomeClient *client, RBShell *shell);
+static void rb_shell_session_init (RBShell *shell);
+
+
 static const GtkTargetEntry target_table[] = { { "text/uri-list", 0,0 } };
 
 #define CMD_PATH_VIEW_SMALLDISPLAY "/commands/ToggleSmallDisplay"
@@ -253,6 +272,8 @@ struct RBShellPrivate
 
 	GList *sources;
 
+	gboolean play_queued;
+
 	gboolean db_dirty;
 	guint async_state_save_id;
 
@@ -285,8 +306,6 @@ struct RBShellPrivate
 
 	char *cached_title;
 	char *cached_duration;
-
-	gulong song_changed_cb_id;
 };
 
 static BonoboUIVerb rb_shell_verbs[] =
@@ -382,6 +401,8 @@ rb_shell_init (RBShell *shell)
 	gnome_window_icon_set_default_from_file (file);
 	g_free (file);
 	
+        rb_shell_session_init (shell);
+
 	eel_gconf_monitor_add (CONF_PREFIX);
 
 }
@@ -498,6 +519,7 @@ idle_do_action (struct RBShellAction *data)
 	char *unquoted = NULL;
 	
 	GDK_THREADS_ENTER ();
+	rb_debug ("entering idle_do_action");
 
 	data->tries++;
 
@@ -507,31 +529,41 @@ idle_do_action (struct RBShellAction *data)
 
 	if (entry) {
 		switch (data->type) {
-		case RB_SHELL_ACTION_JUMP:
-			rb_shell_jump_to_entry (data->shell, entry);
-			break;
 		case RB_SHELL_ACTION_PLAY:
+			rb_debug ("doing play action");
 			rb_shell_play_entry (data->shell, entry);
+			/* fall through */ 
+		case RB_SHELL_ACTION_JUMP:
+			rb_debug ("doing jump action");
+			rb_shell_jump_to_entry (data->shell, entry);
 			break;
 		}
 	} else if (data->tries < 4) {
+		rb_debug ("entry not added yet, queueing retry");
 		g_timeout_add (500 + data->tries*200, (GSourceFunc) idle_do_action, data);
-		return TRUE;
+		goto out_unlock;
 	} else
 		g_warning ("No entry %s in db", data->uri);
 
-	GDK_THREADS_LEAVE ();
+	data->shell->priv->play_queued = FALSE;
 
 	g_free (unquoted);
 	g_free (data->uri);
 	g_free (data);
+
+ out_unlock:
+	GDK_THREADS_LEAVE ();
 	return FALSE;
 }
 
 static void
 rb_shell_queue_jump (RBShell *shell, const char *uri)
 {
-	struct RBShellAction *data = g_new0 (struct RBShellAction, 1);
+	struct RBShellAction *data;
+
+	rb_debug ("queueing jump");
+
+	data = g_new0 (struct RBShellAction, 1);
 	data->shell = shell;
 	data->uri = g_strdup (uri);
 	data->type = RB_SHELL_ACTION_JUMP;
@@ -541,7 +573,15 @@ rb_shell_queue_jump (RBShell *shell, const char *uri)
 static void
 rb_shell_queue_play (RBShell *shell, const char *uri)
 {
-	struct RBShellAction *data = g_new0 (struct RBShellAction, 1);
+	struct RBShellAction *data;
+
+	if (shell->priv->play_queued) {
+		rb_debug ("file already queued for playback");
+		return;
+	}
+	rb_debug ("queueing play");
+
+	data = g_new0 (struct RBShellAction, 1);
 	data->shell = shell;
 	data->uri = g_strdup (uri);
 	data->type = RB_SHELL_ACTION_PLAY;
@@ -554,6 +594,8 @@ rb_shell_corba_quit (PortableServer_Servant _servant,
 {
 	RBShell *shell = RB_SHELL (bonobo_object (_servant));
 
+	rb_debug ("corba quit");
+
 	GDK_THREADS_ENTER ();
 
 	rb_shell_quit (shell);
@@ -569,6 +611,8 @@ rb_shell_corba_handle_file (PortableServer_Servant _servant,
 	RBShell *shell = RB_SHELL (bonobo_object (_servant));
 	RBPlaylist *parser;
 
+	rb_debug ("handling uri: %s", uri);
+
 	GnomeVFSURI *vfsuri = gnome_vfs_uri_new (uri);
 	if (!vfsuri) {
 		rb_error_dialog (_("Unable to parse URI \"%s\"\n"), uri);
@@ -577,9 +621,12 @@ rb_shell_corba_handle_file (PortableServer_Servant _servant,
 
 	parser = rb_playlist_new ();
 	if (rb_playlist_can_handle (uri)) {
+		rb_debug ("parsing uri as playlist: %s", uri);
 		uri = rb_playlist_manager_parse_file (shell->priv->playlist_manager, uri);
-	} else 
+	} else {
+		rb_debug ("async adding uri: %s", uri);
 		rhythmdb_add_uri_async (shell->priv->db, uri);
+	}
 
 	rb_shell_queue_play (shell, uri);
 	g_object_unref (G_OBJECT (parser));
@@ -592,6 +639,7 @@ rb_shell_corba_add_to_library (PortableServer_Servant _servant,
 {
 	RBShell *shell = RB_SHELL (bonobo_object (_servant));
 
+	rb_debug ("async adding uri: %s", uri);
 	rhythmdb_add_uri_async (shell->priv->db, uri);
 }
 
@@ -601,13 +649,15 @@ rb_shell_corba_grab_focus (PortableServer_Servant _servant,
 {
 	RBShell *shell = RB_SHELL (bonobo_object (_servant));
 
+	rb_debug ("grabbing focus");
 	gtk_window_present (GTK_WINDOW (shell->priv->window));
 	gtk_widget_grab_focus (shell->priv->window);
 }
 
 enum PlayerProperties {
 	PROP_VISIBILITY,
-	PROP_SONG
+	PROP_SONG,
+	PROP_SHUFFLE,
 };
 
 
@@ -682,6 +732,14 @@ shell_pb_get_prop (BonoboPropertyBag *bag,
 		BONOBO_ARG_SET_BOOLEAN (arg, FALSE);
 		break;
 
+	case PROP_SHUFFLE:
+	{
+		char *play_order;
+		g_object_get (G_OBJECT (player), "play-order", &play_order, NULL);
+		BONOBO_ARG_SET_BOOLEAN (arg, !strcmp ("shuffle", play_order));
+		break;
+	}
+
 	case PROP_SONG: {
 		GNOME_Rhythmbox_SongInfo *ret_val;
 		ret_val = get_song_info_from_player (shell);
@@ -729,29 +787,27 @@ rb_shell_corba_get_player_properties (PortableServer_Servant _servant,
 {	
 	RBShell *shell = RB_SHELL (bonobo_object (_servant));
 
+	rb_debug ("getting player properties");
 	if (shell->priv->pb == NULL) {
-		gchar *params_to_map[] = {"repeat", "shuffle", "playing", 
-					  NULL};
+		gchar *params_to_map[] = {"repeat", "play-order", "playing"}; 
 		GParamSpec **params;
-		int length = 0;
 		int i = 0;
+		int total = 0;
 
 		shell->priv->pb = bonobo_property_bag_new (shell_pb_get_prop, 
 							   shell_pb_set_prop, 
 							   shell);
 		
 		
-		/* Map RBShellPlayer properties to bonobo properties */
-		while (params_to_map[length] != NULL) {
-			length++;
-		}
-		params = malloc (length * sizeof (GParamSpec *));
-		for (i = 0; i < length; i++) {
-			params[i] = g_object_class_find_property (G_OBJECT_CLASS (RB_SHELL_PLAYER_GET_CLASS (shell->priv->player_shell)), params_to_map[i]);
+		params = malloc (G_N_ELEMENTS (params_to_map) * sizeof (GParamSpec *));
+		for (i = 0; i < G_N_ELEMENTS (params_to_map); i++) {
+			params[total] = g_object_class_find_property (G_OBJECT_CLASS (RB_SHELL_PLAYER_GET_CLASS (shell->priv->player_shell)), params_to_map[i]);
+			if (params[total])
+				total++;
 		}
 		bonobo_property_bag_map_params (shell->priv->pb,
 						G_OBJECT (shell->priv->player_shell),
-						(const GParamSpec **)params, length);
+						(const GParamSpec **)params, total);
 
 
 		/* Manually install the other properties */
@@ -759,6 +815,11 @@ rb_shell_corba_get_player_properties (PortableServer_Servant _servant,
 					 PROP_VISIBILITY, BONOBO_ARG_BOOLEAN, NULL, 
 					 _("Whether the main window is visible"), 0);
 
+		/* Manually install the other properties */
+		bonobo_property_bag_add (shell->priv->pb, "shuffle", 
+					 PROP_SHUFFLE, BONOBO_ARG_BOOLEAN, NULL, 
+					 _("Whether shuffle is enabled"), 0);
+
 		bonobo_property_bag_add (shell->priv->pb, "song", 
 					 PROP_SONG, TC_GNOME_Rhythmbox_SongInfo, NULL, 
 					 _("Properties for the current song"), 0);
@@ -775,6 +836,7 @@ rb_shell_corba_playpause (PortableServer_Servant _servant,
 			  CORBA_Environment *ev)
 {
 	RBShell *shell = RB_SHELL (bonobo_object (_servant));
+	rb_debug ("got playpause");
 	rb_shell_player_playpause (shell->priv->player_shell);
 }
 
@@ -784,6 +846,7 @@ rb_shell_corba_select (PortableServer_Servant _servant,
 		       CORBA_Environment *ev)
 {
 	RBShell *shell = RB_SHELL (bonobo_object (_servant));
+	rb_debug ("got select");
 	rb_shell_queue_jump (shell, uri);
 }
 
@@ -793,6 +856,7 @@ rb_shell_corba_play (PortableServer_Servant _servant,
 		     CORBA_Environment *ev)
 {
 	RBShell *shell = RB_SHELL (bonobo_object (_servant));
+	rb_debug ("got play");
 	rb_shell_queue_play (shell, uri);
 }
 
@@ -801,6 +865,7 @@ rb_shell_corba_next (PortableServer_Servant _servant,
 		     CORBA_Environment *ev)
 {
 	RBShell *shell = RB_SHELL (bonobo_object (_servant));
+	rb_debug ("got next");
 	rb_shell_player_do_next (shell->priv->player_shell);
 }
 
@@ -809,6 +874,7 @@ rb_shell_corba_previous (PortableServer_Servant _servant,
 			 CORBA_Environment *ev)
 {
 	RBShell *shell = RB_SHELL (bonobo_object (_servant));
+	rb_debug ("got previous");
 	rb_shell_player_do_previous (shell->priv->player_shell);
 }
 
@@ -817,6 +883,7 @@ rb_shell_corba_get_playing_time (PortableServer_Servant _servant,
 				 CORBA_Environment *ev)
 {
 	RBShell *shell = RB_SHELL (bonobo_object (_servant));
+	rb_debug ("got playing time");
 	return rb_shell_player_get_playing_time (shell->priv->player_shell);
 }
 
@@ -825,6 +892,8 @@ rb_shell_corba_set_playing_time (PortableServer_Servant _servant,
 				 CORBA_long time, CORBA_Environment *ev)
 {
 	RBShell *shell = RB_SHELL (bonobo_object (_servant));
+	rb_debug ("got set playing time");
+	rb_debug ("got milk?");
 	rb_shell_player_set_playing_time (shell->priv->player_shell, time);
 }
 
@@ -848,13 +917,30 @@ rb_shell_entry_changed_cb (GObject *object, GParamSpec *pspec, RBShell *shell)
 {
 	GNOME_Rhythmbox_SongInfo *song_info;
 	BonoboArg *arg;
-	
+
 	g_assert (strcmp (pspec->name, "playing-entry") == 0);
 	song_info = get_song_info_from_player (shell);
 	arg = bonobo_arg_new (TC_GNOME_Rhythmbox_SongInfo);
 	(GNOME_Rhythmbox_SongInfo*)arg->_value = song_info;
 	shell_notify_pb_changes (shell, "song", arg);
 	/* FIXME: arg should be released somehow */
+
+#ifdef WITH_DASHBOARD
+	if (song_info) {
+        	char *cluepacket;
+        	/* Send cluepacket to dashboard */
+        	cluepacket =
+			dashboard_build_cluepacket_then_free_clues ("Music Player",
+							    	TRUE, 
+							    	"", 
+							    	dashboard_build_clue (song_info->title, "song_title", 10),
+							    	dashboard_build_clue (song_info->artist, "artist", 10),
+							    	dashboard_build_clue (song_info->album, "album", 10),
+							    	NULL);
+       		dashboard_send_raw_cluepacket (cluepacket);
+       		g_free (cluepacket);
+	}
+#endif //WITH_DASHBOARD
 }
 
 void
@@ -1416,17 +1502,17 @@ rb_shell_select_source (RBShell *shell,
 
 	rb_debug ("selecting source %p", source);
 	
-	if (shell->priv->song_changed_cb_id != 0) {
+	if (shell->priv->selected_source) {
 		view = rb_source_get_entry_view (shell->priv->selected_source);
-		g_signal_handler_disconnect (view, 
-					     shell->priv->song_changed_cb_id);
+		g_signal_handlers_disconnect_by_func (view, 
+		                                      G_CALLBACK (rb_shell_entry_changed_cb),
+						      shell);
 	}
 	shell->priv->selected_source = source;
 	
 	view = rb_source_get_entry_view (shell->priv->selected_source);
-	shell->priv->song_changed_cb_id = 
-		g_signal_connect (view, "notify::playing-entry", 
-				  (GCallback)rb_shell_entry_changed_cb, shell);
+	g_signal_connect (view, "notify::playing-entry", 
+			  G_CALLBACK(rb_shell_entry_changed_cb), shell);
 
 	/* show source */
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (shell->priv->notebook),
@@ -2091,3 +2177,43 @@ tray_deleted_cb (GtkWidget *win, GdkEventAny *event, RBShell *shell)
 	g_signal_connect (G_OBJECT (shell->priv->tray_icon), "delete_event",
 			  G_CALLBACK (tray_deleted_cb), shell);
 }
+
+static void 
+session_die_cb (GnomeClient *client, 
+                RBShell *shell)
+{
+        rb_debug ("session die");
+        rb_shell_quit (shell);
+}
+
+static gboolean
+save_yourself_cb (GnomeClient *client, 
+                  gint phase,
+                  GnomeSaveStyle save_style,
+                  gboolean shutdown,
+                  GnomeInteractStyle interact_style,
+                  gboolean fast,
+                  RBShell *shell)
+{
+        rb_debug ("session save yourself");
+        rb_shell_sync_state (shell);
+        return TRUE;
+}
+
+static void
+rb_shell_session_init (RBShell *shell)
+{
+        GnomeClient *client;
+
+        client = gnome_master_client ();
+
+        g_signal_connect (G_OBJECT (client), 
+                          "save_yourself",
+                          G_CALLBACK (save_yourself_cb),
+                          shell);
+
+        g_signal_connect (G_OBJECT (client),
+                          "die",
+                          G_CALLBACK (session_die_cb),
+                          shell);
+}
